﻿/*
	Antenna House PDF Tool API V7.0
	C++ Interface sample program

	概要：画像の入れ替え

	Copyright 2021 Antenna House,Inc.
*/

#include <PdfTk.h>
#include <stdio.h>

using namespace PdfTk;
using namespace std;

bool replaceImage(PtlContent& content, const string& pdfPath);
bool replaceImageWithImage(PtlEditImage& elemImage, const string& pdfPath);
bool replaceImageWIthPdf(PtlEditElements& elems, int index, PtlEditImage& elemImage, const string& pdfPath);

int main(int argc, char* argv[])
{
	if (argc != 4)
	{
		printf("usage: ReplaceImage.exe in-pdf-file out-pdf-file image-file \n");
		return 1;
	}
	try {
		PtlOption option;
		option.setUnit(PtlOption::UNIT_PT); // Matirixを扱うため

		PtlParamInput input(argv[1]);
		PtlParamOutput output(argv[2]);
		std::string imgPath = argv[3];

		PtlPDFDocument doc;
		// PDFファイルをロードします。
		doc.load(input);
		PtlPages& pages = doc.getPages();
		int pageCount = pages.getCount();
        for (int i = 0; i < pageCount; ++i)
        {
			PtlPage page = pages.get(i);
			PtlContent& contentPage = page.getContent();
            {
                // ページコンテントをパースして画像を置き換える
                if (replaceImage(contentPage, imgPath))
                {
                    page.setContent(contentPage);   // ページコンテントを置き換える
                }
            }
		}
		doc.save(output);
		printf("完了!\n");
	}
	catch (const PtlException& e)
	{
		fprintf(stderr, "Error code : %d\n %s\n", e.getErrorCode(), e.getErrorMessage().c_str());
		return 1;
	}
}

bool replaceImage(PtlContent& content, const string& imageParh)
{
	bool ret = false;
    try
    {
        PtlEditElements& elems = content.getEditElements();

        if (elems.isEmpty()) return false;
        int elementcount = elems.getCount();
        for (int i = 0; i < elementcount; ++i)
        {
            PtlEditElement& elem = elems.get(i);
            switch (elem.getType())
            {
            case PtlEditElement::TYPE_FORMXOBJECT:
                {
                    PtlEditFormXObject& elemForm = (PtlEditFormXObject&)elem;
                    PtlContent& contentForm = elemForm.getContent();            // FormXObjectコンテント取得
                    // FormXObjectコンテントをパースして画像を置き換える
                    if (replaceImage(contentForm, imageParh))
                    {
                        elemForm.setContent(contentForm);   // FormXObjectコンテントを置き換える
                        // trueは返さない
                        // 何故ならPtlEditFormXObjectでsetContentしたので上位でsetContentする必要がない
                    }
                }
                break;
            case PtlEditElement::TYPE_MCCONTAINER:
                {
                    PtlEditMCContainer& elemContainer = (PtlEditMCContainer&)elem;
                    PtlContent& contentContainer = elemContainer.getContent();    // マークコンテント取得
                    {
                        // マークコンテントをパースして画像を置き換える
                        if (replaceImage(contentContainer, imageParh))
                        {
                            ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                        }
                    }
                }
                break;
            case PtlEditElement::TYPE_IMAGE:
                {
                    PtlEditImage& elemImage = (PtlEditImage&)elem;
                    if (imageParh.substr(imageParh.length()-3, 3) == "pdf")
                    {
                        // FormXObjectを挿入して画像を削除する
                        if (replaceImageWIthPdf(elems, i, elemImage, imageParh))
                        {
                            ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                        }
                    }
                    else
                    {
                        // PtlEditImageの画像を直接置き換える
                        if (replaceImageWithImage(elemImage, imageParh))
                        {
                            ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                        }
                    }
                }
                break;
            default:
                break;
            }
        }
    }
    catch (PtlException e)
    {
        fprintf(stderr, "Error code : %d\n %s\n", e.getErrorCode(), e.getErrorMessage().c_str());
		return 1;
    }
    return ret;
}

// PtlEditImageの画像を直接置き換える
bool replaceImageWithImage(PtlEditImage& elemImage, const string& imagePath)
{
    try
    {
        PtlParamInput input(imagePath.c_str());
        PtlParamEditImage paramEditImage;
        {
            paramEditImage.setImageStream(input);
            paramEditImage.setUseOriginalJpegQuality(true);
            elemImage.setImage(paramEditImage); // PtlParamEditImageに設定した画像で置き換え
        }
        return true;    // Elementが所属するPtlPage若しくはPtlEditFormXObjectでsetContentが必要なのでtrueを返す
    }
    catch (PtlException e)
    {
        fprintf(stderr, "Error code : %d\n %s\n", e.getErrorCode(), e.getErrorMessage().c_str());
		return 1;
    }
    // 何か問題が起こった
    return false;
}

// 新しいPtlEditFormXObjectにページを設定したものを挿入し、元のPtlEditImageは削除する
bool replaceImageWIthPdf(PtlEditElements& elems, int index, PtlEditImage& elemImage, const string& pdfPath)
{
    try
    {
        PtlParamInput input(pdfPath.c_str());
        PtlPDFDocument doc;
        {
            doc.load(input);
            PtlPages& pages = doc.getPages();
            PtlPage page = pages.get(0);             // 1ページ目を使用する
            PtlRect viewRect = page.getViewBox();    // ページサイズ

            float widthPage = viewRect.getRight() - viewRect.getLeft();     // ページ幅
            float heightPage = viewRect.getTop() - viewRect.getBottom();    // ページ高さ
            PtlEditFormXObject form(page);          // ページを元にPtlEditFormXObjectを作成
            PtlTransMatrix matImage = elemImage.getMatrix();                    // ページ若しくはFormXObject内での実Matrixを取得
            PtlTransMatrix matRate(1 / widthPage, 0, 0, 1 / heightPage, 0, 0);  // 元の画像が表示されていたサイズに合わせるためのMatrix
            PtlTransMatrix matForm = matImage.concat(matRate);              // 元イメージのMatrixとページサイズよりFormXObjectのMatrixを決定
            {
                if (elemImage.hasClip())
                {
                    PtlEditClip clip = elemImage.getClip();     // PtlEditImageが持っているクリッピングを取得
                    PtlEditClip clipNew = clip.cloneCopy();
                    {
                        form.setClip(clipNew);                  // 元のElementが持っていたクリッピングを設定(setMatrixより先に行うこと)
                    }
                }
                form.setMatrix(matForm);
                elems.insert(index, form);              // 新しいPtlEditImageを挿入
                elems.remove(index + 1);                // 元のElementを削除
            }
        }
        return true;    // Elementが所属するPtlPage若しくはPtlEditFormXObjectでsetContentが必要なのでtrueを返す
    }
    catch (PtlException e)
    {
        fprintf(stderr, "Error code : %d\n %s\n", e.getErrorCode(), e.getErrorMessage().c_str());
		return 1;
    }
    // 何か問題が起こった
    return false;
}
