﻿/*
	Antenna House PDF Tool API V7.0
	.NET Interface sample program

	概要：画像の入れ替え

	Copyright 2021 Antenna House,Inc.
*/

using System;
using PdfTkNet;

namespace ReplaceImage
{
    class ReplaceImage
    {
        static void Main(string[] args)
        {
            if (args.Length < 3)
            {
                Console.WriteLine("usage: ReplaceImage.exe in-pdf-file out-pdf-file image-file");
                return;
            }
            try
            {
                using (PtlOption option = new PtlOption())
                using (PtlParamInput input = new PtlParamInput(args[0]))
                using (PtlParamOutput output = new PtlParamOutput(args[1]))
                using (PtlPDFDocument doc = new PtlPDFDocument())
                {
                    option.setUnit(PtlOption.UNIT.UNIT_PT); // Matirixを扱うため

                    // PDFファイルをロードします。
                    doc.load(input);

                    using (PtlPages pages = doc.getPages())
                    {
                        int pageCount = pages.getCount();
                        for (int i = 0; i < pageCount; ++i)
                        {
                            using (PtlPage page = pages.get(i))
                            using (PtlContent contentPage = page.getContent())
                            {
                                // ページコンテントをパースして画像を置き換える
                                if (replaceImage(contentPage, args[2]))
                                {
                                    page.setContent(contentPage);   // ページコンテントを置き換える
                                }
                            }
                        }
                    }
                    doc.save(output);
                }
                Console.WriteLine("完了!");
            }
            catch (PtlException pex)
            {
                Console.WriteLine(pex.getErrorCode() + " : " + pex.getErrorMessageJP());
                pex.Dispose();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        static bool replaceImage(PtlContent content, string imageParh)
        {
            bool ret = false;
            try
            {
                using (PtlEditElements elems = content.getEditElements())
                {
                    if (elems.isEmpty()) return false;
                    int elementcount = elems.getCount();
                    for (int i = 0; i < elementcount; ++i)
                    {
                        using (PtlEditElement elem = elems.get(i))
                        {
                            switch (elem.getType())
                            {
                                case PtlEditElement.ELEMENT_TYPE.TYPE_FORMXOBJECT:
                                    {
                                        PtlEditFormXObject elemForm = elem as PtlEditFormXObject;
                                        using (PtlContent contentForm = elemForm.getContent())  // FormXObjectコンテント取得
                                        {
                                            // FormXObjectコンテントをパースして画像を置き換える
                                            if (replaceImage(contentForm, imageParh))
                                            {
                                                elemForm.setContent(contentForm);   // FormXObjectコンテントを置き換える
                                                // trueは返さない
                                                // 何故ならPtlEditFormXObjectでsetContentしたので上位でsetContentする必要がない
                                            }
                                        }
                                    }
                                    break;
                                case PtlEditElement.ELEMENT_TYPE.TYPE_MCCONTAINER:
                                    {
                                        PtlEditMCContainer elemContainer = elem as PtlEditMCContainer;
                                        using (PtlContent contentContainer = elemContainer.getContent())    // マークコンテント取得
                                        {
                                            // マークコンテントをパースして画像を置き換える
                                            if (replaceImage(contentContainer, imageParh))
                                            {
                                                ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                                            }
                                        }
                                    }
                                    break;
                                case PtlEditElement.ELEMENT_TYPE.TYPE_IMAGE:
                                    {
                                        PtlEditImage elemImage = elem as PtlEditImage;
                                        if (imageParh.EndsWith("pdf"))
                                        {
                                            // FormXObjectを挿入して画像を削除する
                                            if (replaceImageWIthPdf(elems, i, elemImage, imageParh))
                                            {
                                                ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                                            }
                                        }
                                        else
                                        {
                                            // PtlEditImageの画像を直接置き換える
                                            if (replaceImageWithImage(elemImage, imageParh))
                                            {
                                                ret = true;     // 上位のPtlPageかPtlEditFormXObjectでsetContentが必要
                                            }
                                        }
                                    }
                                    break;
                                default:
                                    break;
                            }
                        }
                    }
                }
            }
            catch (PtlException e)
            {
                Console.WriteLine(string.Format("PtlException Error code : {0}  {1}", e.getErrorCode(), e.getErrorMessage()));
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            return ret;
        }

        // PtlEditImageの画像を直接置き換える
        static bool replaceImageWithImage(PtlEditImage elemImage, string imageParh)
        {
            try
            {
                using (PtlParamInput input = new PtlParamInput(imageParh))
                using (PtlParamEditImage paramEditImage = new PtlParamEditImage())
                {
                    paramEditImage.setImageStream(input);
                    paramEditImage.setUseOriginalJpegQuality(true);
                    elemImage.setImage(paramEditImage); // PtlParamEditImageに設定した画像で置き換え
                }
                return true;    // Elementが所属するPtlPage若しくはPtlEditFormXObjectでsetContentが必要なのでtrueを返す
            }
            catch (PtlException e)
            {
                Console.WriteLine(string.Format("PtlException Error code : {0}  {1}", e.getErrorCode(), e.getErrorMessage()));
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            // 何か問題が起こった
            return false;
        }

        // 新しいPtlEditFormXObjectにページを設定したものを挿入し、元のPtlEditImageは削除する
        static bool replaceImageWIthPdf(PtlEditElements elems, int index, PtlEditImage elemImage, string pdfPath)
        {
            try
            {
                using (PtlParamInput input = new PtlParamInput(pdfPath))
                using (PtlPDFDocument doc = new PtlPDFDocument())
                {
                    doc.load(input);

                    using (PtlPages pages = doc.getPages())
                    using (PtlPage page = pages.get(0))             // 1ページ目を使用する
                    using (PtlRect viewRect = page.getViewBox())    // ページサイズ
                    {
                        float widthPage = viewRect.getRight() - viewRect.getLeft();     // ページ幅
                        float heightPage = viewRect.getTop() - viewRect.getBottom();    // ページ高さ
                        using (PtlEditFormXObject form = new PtlEditFormXObject(page))  // ページを元にPtlEditFormXObjectを作成
                        using (PtlTransMatrix matImage = elemImage.getMatrix())         // ページ若しくはFormXObject内での実Matrixを取得
                        using (PtlTransMatrix matRate = new PtlTransMatrix(1 / widthPage, 0, 0, 1 / heightPage, 0, 0))  // 元の画像が表示されていたサイズに合わせるためのMatrix
                        using (PtlTransMatrix matForm = matImage.concat(matRate))       // 元イメージのMatrixとページサイズよりFormXObjectのMatrixを決定
                        {
                            if (elemImage.hasClip())
                            {
                                using (PtlEditClip clip = elemImage.getClip())          // PtlEditImageが持っているクリッピングを取得
                                using (PtlEditClip clipNew = clip.cloneCopy())
                                {
                                    form.setClip(clipNew);                  // 元のElementが持っていたクリッピングを設定(setMatrixより先に行うこと)
                                }
                            }
                            form.setMatrix(matForm);
                            elems.insert(index, form);              // 新しいPtlEditImageを挿入
                            elems.remove(index + 1);                // 元のElementを削除
                        }
                    }
                }
                return true;    // Elementが所属するPtlPage若しくはPtlEditFormXObjectでsetContentが必要なのでtrueを返す
            }
            catch (PtlException e)
            {
                Console.WriteLine(string.Format("PtlException Error code : {0}  {1}", e.getErrorCode(), e.getErrorMessage()));
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            // 何か問題が起こった
            return false;
        }

    }
}
